
var ShowPage = function (presenter) {
	this.presenter = presenter;

	this.current_base = 0;
	this.current_chapter = 0;
	this.current_frame = 0;

	this.resize();
};

ShowPage.prototype.go_chapter = function(base_id, chapter_id) {
	this.current_base = base_id;
	this.current_chapter = chapter_id;
	this.current_frame = 0;
	this.update_display();
};

ShowPage.prototype.show = function() {
	// 設定: テキストを表示する | しない
	$('#show-message').css('display', $('#menu-setting-text').val() == 1 ? 'block' : 'none');

	// タイマー
	this.next_frame_timer_start();

	$('#showpage').css('display', 'block');
};

ShowPage.prototype.hide = function() {
	// タイマーを無効
	this.next_frame_timer_stop();

	$('#showpage').css('opacity', 1).velocity({
		opacity: 0
	}, {
		duration: 400,
		easing: 'swing',
		complete: function() {
			$(this).css('display', 'none').css('opacity', 1);
		}
	})
	//$('#showpage').css('display', 'none');
};

ShowPage.prototype.resize = function(p_size, p_animate) {
	var size = p_size || get_aspected_size();
	var animate = p_animate || false;
	$('#showpage').velocity({
		'width' : size.width,
		'height': size.height,
		'top'   : size.top,
		'left'  : size.left
	}, {
		'duration': animate ? 300 : 0
	});
	$('#overlay').css('width', size.width).css('height', size.height).css('top', size.top).css('left', size.left);
};

ShowPage.prototype.next_frame_timer_start = function() {
	var ins = this;
	if (this.next_frame_timer && this.next_frame_timer !== false) {
        clearTimeout(this.next_frame_timer);
    }

	var timer_size = parseInt($('#menu-setting-automode').val()) * 1000;
	if (timer_size > 0) {
		this.next_frame_timer = setTimeout(function() {
			ins.next_frame();
		}, timer_size);
	}
};

ShowPage.prototype.next_frame_timer_stop = function() {
	if (this.next_frame_timer && this.next_frame_timer !== false) {
        clearTimeout(this.next_frame_timer);
    }
};

ShowPage.prototype.next_frame = function() {
	var current_base = data[this.current_base];
	var chapter_item = current_base.chapters[this.current_chapter];

	var current_base_frames_count = 0;
	var next_image_frame = 0;

	this.next_frame_timer_start();

	// 設定: テキストを表示する
	if ($('#menu-setting-text').val() == 1) {

		for (var i = 0; i < chapter_item.images.length; i++) {
			var image_item = chapter_item.images[i];
			current_base_frames_count += image_item.contents.length;
		}

		// 現在の基本で大丈夫？
		if (this.current_frame + 1 < current_base_frames_count) {
			this.current_frame++;
			this.update_display();
		}

		// 次のチャプタがあればそこへ
		else if (this.current_chapter + 1 < current_base.chapters.length) {
			this.current_chapter++;
			this.current_frame = 0;
			this.update_display();
		}

		// 次の基本があればそこへ
		else if (this.current_base + 1 < data.length) {
			this.current_base++;
			this.current_chapter = 0;
			this.current_frame = 0;
			this.blackout_display(undefined, function(self){
				self.update_display();
			});
		}

		// 次がなければメニューに戻る
		else {
			this.presenter.to_menu_page();
		}
	}

	// 設定: テキストを表示しない
	else {

		next_frame = -1;
		for (var i = 0; i < chapter_item.images.length; i++) {
			if (this.current_frame < current_base_frames_count) {
				next_frame = current_base_frames_count;
				break;
			}

			var image_item = chapter_item.images[i];
			current_base_frames_count += image_item.contents.length;
		}

		// 現在の基本で大丈夫？
		if (next_frame >= 0) {
			this.current_frame = next_frame;
			this.update_display();
		}

		// 次のチャプタがあればそこへ
		else if (this.current_chapter + 1 < current_base.chapters.length) {
			this.current_chapter++;
			this.current_frame = 0;
			this.update_display();
		}

		// 次の基本があればそこへ
		else if (this.current_base + 1 < data.length) {
			this.current_base++;
			this.current_chapter = 0;
			this.current_frame = 0;
			this.update_display();
		}

		// 次がなければメニューに戻る
		else {
			this.presenter.to_menu_page();
		}
	}
};

ShowPage.prototype.update_display = function() {
	var current_base = data[this.current_base];
	var chapter_item = current_base.chapters[this.current_chapter];
	var frame = 0;
	var item, current_image, next_image, prev_image;

	for (var i = 0; i < chapter_item.images.length; i++) {

		var image_item = chapter_item.images[i];

		if (!item) {
			for (var j = 0; j < image_item.contents.length; j++) {
				var content_item = image_item.contents[j];
				if (frame == this.current_frame) {
					item = content_item;
					current_image = image_item;
					break;
				}
				frame++;
			}
		} else {
			next_image = image_item;
			break;
		}

		if (item) {
			continue;
		}

		prev_image = image_item;
	}

	if (current_image) {
		$('#show-current-image').attr('src', 'resources/images/' + current_image.file + '.png');
	}
	if (next_image) {
		$('#show-next-image').attr('src', 'resources/images/' + next_image.file + '.png');
	}
	if (prev_image) {
		$('#show-prev-image').attr('src', 'resources/images/' + prev_image.file + '.png');
	}

	// アニメーション用タイマーがついていれば無効にする
	if (this.frame_animate_timer && this.frame_animate_timer !== false) {
		clearTimeout(this.frame_animate_timer);
	}

	// フレームの表示処理
	if (item.text) {
		$('#show-message').css('visibility', 'visible');
		$('#show-message').text(item.text);
	} else if (item.wait) {
		var ins = this;
		$('#show-message').css('visibility', 'hidden');
		this.frame_animate_timer = setTimeout(function() {
			ins.next_frame();
		}, item.wait);
	} else if (item.fade) {
		var ins = this;
		var col = item.color === undefined ? 'white' : item.color;
		var opa = item.opacity === undefined ? 0.6 : item.opacity
		this.colorout_display(col, function() {
			ins.next_frame();
		}, undefined, item.fade, opa);
	}
};

ShowPage.prototype.blackout_display = function(callback_complete, callback_middle, time) {
	this.colorout_display('black', callback_complete, callback_middle, time);
};

ShowPage.prototype.whiteout_display = function(callback_complete, callback_middle, time) {
	this.colorout_display('white', callback_complete, callback_middle, time);
};

ShowPage.prototype.colorout_display = function(color, callback_complete, callback_middle, time, opacity) {
	var this_instance = this;

	if (callback_complete === undefined) callback_complete = function() {};
	if (callback_middle === undefined) callback_middle = function() {};
	if (time === undefined) time = 300;
	if (opacity === undefined) opacity = 1;

	$('#overlay').css('display', 'block').css('background-color', color).velocity({
		'opacity': opacity
	}, {
		'duration': time,
		'complete': function() {
			callback_middle(this_instance);
			$('#overlay').velocity({
				'opacity': 0
			}, {
				'duration': time,
				'complete': function() {
					$('#overlay').css('display', 'none');
					callback_complete(this_instance);
				}
			});
		}
	});
};
